#' Perfom uncerrtainty and sensitivity analysis.
#' 
#' @details
#' `analyze_uncertainty()` performs uncertainty analysis and sensitivity
#' analysis. It takes informations from input to first draw to set of values for
#' each parameters.
#'
#' Those two sets are combined in one to create a set uncertainty indicators
#' using `model_npv()`.
#' 
#' Sobol analysis is performed by using a function from `sensitivity` library.
#' Default method is `sobolmartinez`, but this can be changed with the parameter
#' sobol_analysis of the function. Basically, the two sets of values are used
#' by the function chosen.
#' 
#' From the sensitivity function only the value for first and total indices,
#' and the normalized variances of the Conditional Expectations are kept.
#' 
#' Those results and graphical analysis are saved.
#' 
#' @param input list of data.frame, inputs needed to perform analysis.
#' @param sobol_analysis character, method used to perform sensitivity analysis.
#' @param verbose logical, should the function be verbose.
#' 
#' @return List with two slots. The first slot named "u_a" contains a
#' data.frame of uncertainty analysis, the second slot named "s_a" contains the
#' result of the sensitivity analyis.
#'
#' @export
#'  
#' @encoding UTF-8
#' @author Frédéric Grelot, \email{frederic.grelot.1994_cran@m4x.org} 

analyze_uncertainty = function(
    input,
    sobol_analysis = c( "sobolmartinez", "sobol2002", "sobolSalt", "sobol2007", "soboljansen", "soboltouati"),
    verbose = getOption("floodam.cba.sa")[["verbose"]]
) {
    sobol_analysis = match.arg(sobol_analysis)
    msg = sprintf(
        "\t- sentivity analysis is perform using sensitity::%s",
        sobol_analysis
    )

    sobol_analysis = switch(
        sobol_analysis,
        "sobolmartinez" = sensitivity::sobolmartinez,
        "sobol2002" = sensitivity::sobol2002,
        "sobolSalt" = sensitivity::sobolSalt,
        "sobol2007" = sensitivity::sobol2007,
        "soboljansen" = sensitivity::soboljansen,
        "soboltouati" = sensitivity::soboltouati
    )

    nominal = calculate_nominal(input)
	x1 = draw_all_parameter(input, nominal)
	x2 = draw_all_parameter(input, nominal)

	u_a = time_elapsed(
        model_npv,
        rbind(x1, x2[-1, ]),
        indicator = "complete",
        calendar = input[["calendar"]],
        impact_base = set_impact_base(input),
        rate = getOption("floodam.cba.sa")[["rate"]]
    )
 
    write(msg, file = input[["path_log"]], append = TRUE)
    if (isTRUE(verbose)) message(gsub("\t- ", "", msg))

	s_a = time_elapsed(
        sobol_analysis,
        model_npv,
        x1,
        x2,
        nboot = round(input[["n"]] / 20),
        indicator = "npv",
        rate = getOption("floodam.cba.sa")[["rate"]],
        calendar = input[["calendar"]],
        impact_base = set_impact_base(input)
    )

    u_a = as.data.frame(u_a)
    s_a = s_a[c("S", "T", "V")]

	utils::write.csv2(
        u_a,
        file = file.path(input[["path_output"]], "uncertainty.csv"),
        row.names = FALSE
    )
	utils::write.csv2(
        s_a[["S"]],
        file = file.path(input[["path_output"]],"sobol_first.csv")
    )
	utils::write.csv2(
        s_a[["S"]],
        file = file.path(input[["path_output"]],"sobol_total.csv")
    )

    result = list(u_a = u_a, s_a = s_a)

	time_elapsed(analyze_graphic, input, result)
    make_xtable(input)
    time_elapsed(compress, input)

	invisible(result)
}
