#' Calculate weight of flood scenario according to their return period.
#' 
#' @description
#' 
#' `calculate_weight()` calculates the weight that will be used to estimate
#' expected annual damages, depending on either return period or frequency
#' of lood scenarios.
#' 
#' `calculate_weight()` can do this for flood scenarios in any order. This has
#' more interest if x is a named vector.
#' 
#' @param x numeric, values of return period of frequency for flood scenarios.
#' @param type character, 
#' 
#' @return numeric, values of weight for flood scenarios
#' 
#' @export
#' 
#' @examples
#' 
#' x = c(1, 5, 10, 100, 1000)
#' calculate_weight(x)
#' calculate_weight(1/x, "frequency")
#' 
#' names(x) = c("deb", "frequent", "medium", "rare", "extreme")
#' x = rev(c(x, inf = +Inf))
#' calculate_weight(x)
#' calculate_weight(1/x, "frequency")
#' 
#' # When the range of flood scenarios goes from 1 to +Inf weights sum to 1
#' sum(calculate_weight(x))
#' 
#' x = sample(x, 6)
#' calculate_weight(x)
#' 
#' @encoding UTF-8
#' @author Frédéric Grelot, \email{frederic.grelot.1994_cran@m4x.org} 

calculate_weight = function(x, type = c("period", "frequency")) {
    type = match.arg(type)

    if (type == "period") x = 1 / x
    if (is.unsorted(rev(x))) {
        o = order(x, decreasing = TRUE)
        return(calculate_weight(x[o], "frequency")[order(o)])
    }
    result = (c(x[1], x[- length(x)]) - c(x[-1], x[length(x)])) / 2
    names(result) = names(x)
    return(result)
}
