#' Load all data necessary to perform analysis
#' 
#' @details 
#' `load_data()` loads all data necessary to perform analysis. It defines also
#' some key parameters such as where to save outputs, how many draws should be
#' done for uncertainty analysis, what should be the discount rate.
#' 
#' `load_data()` can also make some checks on the format of inputs, to ensure
#' that everything will work fine when applying analysis. If some checks are
#' not passed, a message is given to help correct problems.
#' 
#' @param case character, name of the case study.
#' @param path_input character, path where to find input data.
#' @param path_output character, path where to save output data.
#' @param n integer, path where to save output data.
#' @param type_input character, format of input.
#' @param rate character, parameter to choose discount rate.
#' @param check logical, should input data be checked.
#' @param verbose logical, should the function be verbose.
#' 
#' @return list of all input data and useful paths.
#' 
#' @examples
#' 
#' case = "test"
#' path_input = system.file("extdata", package = "floodam.cba.sa")
#' 
#' # minimum information given
#' input = load_data(case, path_input)
#' 
#' # without any message
#' input = load_data(case, path_input, verbose = FALSE)
#' 
#' # readjusting
#' input = load_data(case, path_input, n = 100)
#' 
#' @export
#'  
#' @encoding UTF-8
#' @author Frédéric Grelot, \email{frederic.grelot.1994_cran@m4x.org}

load_data = function(
    case,
    path_input,
    path_output = getOption("floodam.cba.sa")[["path_output"]],
    n = 1000,
    type_input = getOption("floodam.cba.sa")[["type_input"]],
    rate = getOption("floodam.cba.sa")[["rate"]],
    check = getOption("floodam.cba.sa")[["check"]],
    verbose = getOption("floodam.cba.sa")[["verbose"]]
)
{
    if (is.null(path_output)) path_output = tempdir()
    if (!is.character(case) && length(case) == 1)
        stop("'case' must be a character of length 1")
    if (!is.character(path_input) && length(path_input) == 1)
        stop("'path_input' must be a character of length 1")
    if (!is.character(path_output) && length(path_output) == 1)
        stop("'path_output' must be a character of length 1")

    if (!case %in% dir(path_input)) {
        msg = sprintf(
            "case '%s' not found in path '%s'. Please check.",
            case,
            path_input
        )
        stop(msg)
    }

    path_input = file.path(path_input, case)
    path_output = file.path(path_output, case)
    unlink(path_output, TRUE)
    dir.create(path_output, FALSE, TRUE)
    path_log = file.path(path_output, "information.log")

    result = read_input(path_input, type_input)
	
	if(isTRUE(check)) {
		check_data(result)
		lapply(
            getOption("floodam.cba.sa")[["data"]],
            check_column,
            input = result
        )
		lapply(
            getOption("floodam.cba.sa")[["data"]],
            check_row,
            input = result
        )
	}

    result[["case"]] = case
    result[["path_input"]] = path_input
    result[["path_output"]] = path_output
    result[["path_log"]] = path_log

    n = as.integer(n)
    if (n < 1000) {
        n = as.integer(1000)
        msg = "n too small, modified to 1000"
        if (isTRUE(verbose)) message(msg)
    }
    result[["n"]] = n
    result[["rate"]] = rate

    msg = c(
        sprintf("Case '%s' successfully loaded.", result[["case"]]),
        "Key informations are:",
        sprintf(
            "\t- 'floodam.cba.sa' in version %s is used with:",
            utils::packageVersion("floodam.cba.sa")
        ),
        sprintf("\t\t- n=%s", result[["n"]]),
        sprintf("\t\t- rate=%s", result[["rate"]]),
        sprintf(
            "\t- 'sensitivity' in version %s is used.",
            utils::packageVersion("sensitivity")
        ),
        sprintf("\t- input are taken from '%s'", result[["path_input"]]),
        sprintf("\t- output are saved in '%s'", result[["path_output"]])
    )
    if (isTRUE(verbose)) message(sprintf("%s\n", msg))
    write(msg, file = result[["path_log"]])

    return(invisible(result))
}