#' Read input from a directory from different possible formats
#' 
#' @details
#' 
#' `read_input()` loads all data from a given type in a directory. It does not
#' perform any check on the integrity of the data.
#' 
#' `type_input` controls the way data is imported:
#' \describe{
#'      \item{csv2}{use data.table::fread with dec = "," on all "csv" files
#'      found in path_input. So sep is guessed.}
#'      \item{csv1}{use data.table::fread without sepcification on all "csv"
#'      files found in path_input.. So sep is guessed.}
#'      \item{ods}{use readODS::read_ods on all sheets of the "ods" file found
#'      in path_input.}
#'      \item{xlsx}{use readxl::read_xlsx on all sheets of the "xlsx" file
#'      found in path_input.}
#' }
#' 
#' @param path_input character, path where to find input data.
#' @param type_input character, format of input data. See details.
#' 
#' @return Named list of data.frame of input data found in `path_input`.
#' 
#' @examples
#' 
#' path_input = system.file("extdata", "test", package = "floodam.cba.sa")
#' 
#' # Default to csv2
#' input_default = read_input(path_input)
#' input_csv2 = read_input(path_input, "csv2")
#' identical(input_default, input_csv2)
#'
#' # ods format
#' input_ods = read_input(path_input, "ods")
#' # May not be TRUE because of numeric / integer format
#' identical(input_default, input_ods)
#' all.equal(input_default, input_ods, check.attributes = FALSE)
#' 
#' # xlsx format
#' input_xlsx = read_input(path_input, "xlsx")
#' # May not be TRUE because of numeric / integer format
#' identical(input_default, input_xlsx)
#' all.equal(input_default, input_xlsx, check.attributes = FALSE)
#' 
#' @export
#'  
#' @encoding UTF-8
#' @author Frédéric Grelot, \email{frederic.grelot.1994_cran@m4x.org}

read_input = function(
    path_input,
    type_input = c("csv2", "csv1", "ods", "xlsx")
) {
    # Function to ensure that row.names are characters...
    fun = function(x) {
        result = data.frame(x, row.names = 1)
        row.names(result) = row.names(result)
        return(result)
    }

    type_input = match.arg(type_input)
    ext_input = switch(
        type_input,
        "csv2" = "csv",
        "csv1" = "csv",
        "ods"  = "ods",
        "xlsx" = "xlsx"
    )
    n_input = switch(
        type_input,
        "csv2" = +Inf,
        "csv1" = +Inf,
        "ods"  = 1,
        "xlsx" = 1
    )

    selection = dir(path_input, ext_input, full.names = TRUE)
    if (length(selection) == 0) {
        msg = c(
            sprintf(
                "No '%s' files found in '%s'.",
                ext_input,
                path_input
            ),
            " Please reorganize your data."
        )
        warning(msg)
        return(NULL)
    }
    if (length(selection) > n_input) {
        msg = c(
            sprintf(
                "Two much 'xlsx' files found in '%s'.",
                ext_input,
                path_input
            ),
            " Please reorganize your data."
        )
        warning(msg)
        return(NULL)
    }

    if (type_input == "csv2") {
        result = lapply(selection, data.table::fread, dec = ",")
        input_name = tools::file_path_sans_ext(basename(selection))
    }

    if (type_input == "csv1") {
        result = lapply(selection, data.table::fread, dec = ".")
        input_name = tools::file_path_sans_ext(basename(selection))
    }

    if (type_input == "ods") {
        input_name = readODS::list_ods_sheets(selection)
        result = lapply(
            seq_along(input_name),
            readODS::read_ods,
            path = selection
        )
    }

    if (type_input == "xlsx") {
        input_name = readxl::excel_sheets(selection)
        result = lapply(
            seq_along(input_name),
            readxl::read_xlsx,
            path = selection
        )
    }

    result = lapply(result, fun)
    names(result) = input_name
    return(result)
}
    
